# -*- coding: utf-8 -*-

import shutil
import grokcore.component as grok
from ZODB.interfaces import IBlob
from zope.copy.interfaces import ICopyHook, ResumeCopy


class BlobFileCopyHook(grok.Adapter):
    """A copy hook for IBlobFile objects.
    """
    grok.implements(ICopyHook)
    grok.context(IBlob)

    def __call__(self, toplevel, register):
        register(self._copyBlob)
        raise ResumeCopy

    def _copyBlob(self, translate):
        target = translate(self.context)

        # We init the blob to get a consistent state.
        target.__init__()

        fsrc = self.context.open('r')
        fdst = target.open('w')
        shutil.copyfileobj(fsrc, fdst)
        fdst.close()
        fsrc.close()



# -*- coding: utf-8 -*-

from dolmen.blob import BlobValue

_marker = object()


class BlobProperty(object):
    """Stores the given file data in a blob.
    """

    def __init__(self, field, name=None):
        self.__field = field
        self.__name = name or field.__name__

    def __set__(self, inst, value):
        name = self.__name
        field = self.__field.bind(inst)
        fields = inst.__dict__

        if field.readonly and self.__name in field:
            raise ValueError(self.__name, 'field is readonly')

        if value:
            filename = getattr(value, 'filename', None)
            file = BlobValue(data=value, filename=filename)
        else:
            file = None

        fields[name] = file
        inst._p_changed = True

    def __get__(self, inst, klass):
        if inst is None:
            return self

        value = inst.__dict__.get(self.__name, _marker)
        if value is _marker:
            field = self.__field.bind(inst)
            value = getattr(field, 'default', _marker)
            if value is _marker:
                raise AttributeError(self.__name)
        return value

    def __getattr__(self, name):
        return getattr(self.__field, name)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from os.path import join

name = 'dolmen.blob'
version = '0.5.0'
readme = open(join('src', 'dolmen', 'blob', "README.txt")).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = [
    'ZODB3 >= 3.9.0',
    'dolmen.builtins >= 0.3.1',
    'dolmen.file >= 0.5.1',
    'grokcore.component',
    'grokcore.view',
    'setuptools',
    'zope.component >= 3.9.1',
    'zope.contenttype',
    'zope.copy >= 3.5.0',
    'zope.file',
    'zope.filerepresentation',
    'zope.interface',
    'zope.location >= 3.7.0',
    'zope.mimetype',
    'zope.publisher',
    'zope.schema',
    ]

tests_require = [
    'transaction',
    'zope.container',
    'zope.size',
    'zope.testing',
    'zope.security',
    'zope.traversing',
    'zope.app.testing',
    'zope.app.appsetup',
    ]

setup(name=name,
      version=version,
      description='Dolmen zodb blob handlers',
      long_description=readme + '\n\n' + history,
      keywords='Grok Zope3 CMS Dolmen',
      author='Souheil Chelfouh',
      author_email='trollfot@gmail.com',
      url='',
      license='GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages=['dolmen'],
      include_package_data=True,
      platforms='Any',
      zip_safe=False,
      tests_require=tests_require,
      install_requires=install_requires,
      extras_require={'test': tests_require},
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
      ],
      )



# -*- coding: utf-8 -*-

import grokcore.component as grok
from dolmen.blob import IBlobFile
from zope.filerepresentation.interfaces import IReadFile, IWriteFile


class ReadFileAdapter(grok.Adapter):
    grok.context(IBlobFile)
    grok.implements(IReadFile)

    def size(self):
        return self.context.size

    def read(self):
        # We use the natural getter.
        return self.context.data


class WriteFileAdapter(grok.Adapter):
    grok.context(IBlobFile)
    grok.implements(IWriteFile)

    def write(self, data):
        # We use the natural setter to get the Storage capabilities.
        self.context.data = data
